@@ -50,6 +50,7 @@ namespace Http
 
         bool setupHttps(const QByteArray &certificates, const QByteArray &privateKey);
         void disableHttps();
+        bool isHttps() const;
 
     private slots:
         void dropTimedOutConnection();
@@ -58,10 +59,10 @@ namespace Http
         void incomingConnection(qintptr socketDescriptor) override;
         void removeConnection(Connection *connection);
 
-        IRequestHandler *m_requestHandler;
+        IRequestHandler *m_requestHandler = nullptr;
         QSet<Connection *> m_connections;  // for tracking persistent connections
 
-        bool m_https;
+        bool m_https = false;
         QList<QSslCertificate> m_certificates;
         QSslKey m_key;
     };
