@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2018  Vladimir Golovnev <glassez@yandex.ru>
+ * Copyright (C) 2018-2024  Vladimir Golovnev <glassez@yandex.ru>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,33 +28,34 @@
 
 #pragma once
 
-#include <QObject>
-#include <QVariant>
 #include <QtContainerFwd>
+#include <QObject>
+#include <QString>
+#include <QVariant>
 
-class QString;
-
-struct ISessionManager;
+#include "base/applicationcomponent.h"
 
 using DataMap = QHash<QString, QByteArray>;
 using StringMap = QHash<QString, QString>;
 
-class APIController : public QObject
+struct APIResult
+{
+    QVariant data;
+    QString mimeType;
+    QString filename;
+
+    void clear();
+};
+
+class APIController : public ApplicationComponent<QObject>
 {
     Q_OBJECT
     Q_DISABLE_COPY_MOVE(APIController)
 
-#ifndef Q_MOC_RUN
-#define WEBAPI_PUBLIC
-#define WEBAPI_PRIVATE
-#endif
-
 public:
-    explicit APIController(ISessionManager *sessionManager, QObject *parent = nullptr);
+    explicit APIController(IApplication *app, QObject *parent = nullptr);
 
-    QVariant run(const QString &action, const StringMap &params, const DataMap &data = {});
-
-    ISessionManager *sessionManager() const;
+    APIResult run(const QString &action, const StringMap &params, const DataMap &data = {});
 
 protected:
     const StringMap &params() const;
@@ -64,10 +65,10 @@ protected:
     void setResult(const QString &result);
     void setResult(const QJsonArray &result);
     void setResult(const QJsonObject &result);
+    void setResult(const QByteArray &result, const QString &mimeType = {}, const QString &filename = {});
 
 private:
-    ISessionManager *m_sessionManager;
     StringMap m_params;
     DataMap m_data;
-    QVariant m_result;
+    APIResult m_result;
 };
